@@ -662,7 +662,7 @@ pg_decode_begin_txn_v1(LogicalDecodingContext *ctx, ReorderBufferTXN *txn)
 
 	if (data->include_lsn)
 	{
-		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, txn->end_lsn));
+		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, UInt64GetDatum(txn->end_lsn)));
 
 		appendStringInfo(ctx->out, "%s\"nextlsn\":%s\"%s\",%s", data->ht, data->sp, lsn_str, data->nl);
 
@@ -706,7 +706,7 @@ pg_decode_begin_txn_v2(LogicalDecodingContext *ctx, ReorderBufferTXN *txn)
 
 	if (data->include_lsn)
 	{
-		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, txn->final_lsn));
+		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, UInt64GetDatum(txn->final_lsn)));
 		appendStringInfo(ctx->out, ",\"lsn\":\"%s\"", lsn_str);
 		pfree(lsn_str);
 	}
@@ -740,7 +740,7 @@ pg_decode_commit_txn_v1(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 		elog(DEBUG2, "txn has catalog changes: yes");
 	else
 		elog(DEBUG2, "txn has catalog changes: no");
-	elog(DEBUG2, "my change counter: %lu ; # of changes: %lu ; # of changes in memory: %lu", data->nr_changes, txn->nentries, txn->nentries_mem);
+	elog(DEBUG2, "my change counter: " UINT64_FORMAT " ; # of changes: " UINT64_FORMAT " ; # of changes in memory: " UINT64_FORMAT, data->nr_changes, txn->nentries, txn->nentries_mem);
 	elog(DEBUG2, "# of subxacts: %d", txn->nsubtxns);
 
 	/* Transaction ends */
@@ -780,7 +780,7 @@ pg_decode_commit_txn_v2(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 
 	if (data->include_lsn)
 	{
-		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, commit_lsn));
+		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, UInt64GetDatum(commit_lsn)));
 		appendStringInfo(ctx->out, ",\"lsn\":\"%s\"", lsn_str);
 		pfree(lsn_str);
 	}
@@ -1121,17 +1121,17 @@ pg_decode_change_v1(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 
 	data = ctx->output_plugin_private;
 
-	if (!data->actions.insert)
+	if (change->action == REORDER_BUFFER_CHANGE_INSERT && !data->actions.insert)
 	{
 		elog(DEBUG3, "ignore INSERT");
 		return;
 	}
-	if (!data->actions.update)
+	if (change->action == REORDER_BUFFER_CHANGE_UPDATE && !data->actions.update)
 	{
 		elog(DEBUG3, "ignore UPDATE");
 		return;
 	}
-	if (!data->actions.delete)
+	if (change->action == REORDER_BUFFER_CHANGE_DELETE && !data->actions.delete)
 	{
 		elog(DEBUG3, "ignore DELETE");
 		return;
@@ -1157,6 +1157,7 @@ pg_decode_change_v1(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 	if (list_length(data->filter_tables) > 0)
 	{
 		ListCell	*lc;
+		bool		skip = false;
 
 		foreach(lc, data->filter_tables)
 		{
@@ -1169,10 +1170,18 @@ pg_decode_change_v1(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 					elog(DEBUG2, "\"%s\".\"%s\" was filtered out",
 								((t->allschemas) ? "*" : t->schemaname),
 								((t->alltables) ? "*" : t->tablename));
-					return;
+					skip = true;
 				}
 			}
 		}
+
+		/* table was found */
+		if (skip)
+		{
+			MemoryContextSwitchTo(old);
+			MemoryContextReset(data->context);
+			return;
+		}
 	}
 
 	/* Add tables */
@@ -1200,7 +1209,11 @@ pg_decode_change_v1(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 
 		/* table was not found */
 		if (skip)
+		{
+			MemoryContextSwitchTo(old);
+			MemoryContextReset(data->context);
 			return;
+		}
 	}
 
 	/* Sanity checks */
@@ -1668,7 +1681,7 @@ pg_decode_write_change(LogicalDecodingContext *ctx, ReorderBufferTXN *txn, Relat
 
 	if (data->include_lsn)
 	{
-		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, change->lsn));
+		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, UInt64GetDatum(change->lsn)));
 		appendStringInfo(ctx->out, ",\"lsn\":\"%s\"", lsn_str);
 		pfree(lsn_str);
 	}
@@ -1790,6 +1803,7 @@ pg_decode_change_v2(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 	if (list_length(data->filter_tables) > 0)
 	{
 		ListCell	*lc;
+		bool		skip = false;
 
 		foreach(lc, data->filter_tables)
 		{
@@ -1802,10 +1816,18 @@ pg_decode_change_v2(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 					elog(DEBUG2, "\"%s\".\"%s\" was filtered out",
 								((t->allschemas) ? "*" : t->schemaname),
 								((t->alltables) ? "*" : t->tablename));
-					return;
+					skip = true;
 				}
 			}
 		}
+
+		/* table was found */
+		if (skip)
+		{
+			MemoryContextSwitchTo(old);
+			MemoryContextReset(data->context);
+			return;
+		}
 	}
 
 	/* Add tables */
@@ -1833,7 +1855,11 @@ pg_decode_change_v2(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 
 		/* table was not found */
 		if (skip)
+		{
+			MemoryContextSwitchTo(old);
+			MemoryContextReset(data->context);
 			return;
+		}
 	}
 
 	pg_decode_write_change(ctx, txn, relation, change);
@@ -2015,7 +2041,7 @@ pg_decode_message_v2(LogicalDecodingContext *ctx, ReorderBufferTXN *txn,
 
 	if (data->include_lsn)
 	{
-		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, lsn));
+		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, UInt64GetDatum(lsn)));
 		appendStringInfo(ctx->out, ",\"lsn\":\"%s\"", lsn_str);
 		pfree(lsn_str);
 	}
@@ -2109,7 +2135,7 @@ static void pg_decode_truncate_v1(LogicalDecodingContext *ctx,
 
 	if (data->include_lsn)
 	{
-		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, change->lsn));
+		char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, UInt64GetDatum(change->lsn)));
 		appendStringInfo(ctx->out, "%s%s%s\"lsn\":%s\"%s\",%s", data->ht, data->ht, data->ht, data->sp, lsn_str, data->nl);
 		pfree(lsn_str);
 	}
@@ -2228,7 +2254,7 @@ static void pg_decode_truncate_v2(LogicalDecodingContext *ctx,
 
 		if (data->include_lsn)
 		{
-			char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, change->lsn));
+			char *lsn_str = DatumGetCString(DirectFunctionCall1(pg_lsn_out, UInt64GetDatum(change->lsn)));
 			appendStringInfo(ctx->out, ",\"lsn\":\"%s\"", lsn_str);
 			pfree(lsn_str);
 		}
